import { clone, update } from './immupdate';
export class ObjectWrapper {
    constructor(_value) {
        this._value = _value;
        this._isLiftWrapper = true;
        this.pipe = pipe;
    }
    value() {
        return this._value;
    }
    clone() {
        return new ObjectWrapper(this._clone());
    }
    _clone() {
        return clone(this._value);
    }
    /**
     * Adds a new key/value to this object. This creates a new type.
     * To add a nullable key to an object while preserving its type, use "update()" instead.
     */
    add(key, value) {
        const result = this._clone();
        result[key] = value;
        return new ObjectWrapper(result);
    }
    /**
     * Returns whether this object contains no keys.
     */
    isEmpty() {
        return Object.keys(this.value()).length === 0;
    }
    /**
     * Creates an Array of all this object's keys, in no particular order.
     */
    keys() {
        return this.pipe(o => Object.keys(o));
    }
    /**
     * Maps this Object's values.
     */
    mapValues(mapFunction) {
        return this.toArray().fold({}, (result, [key, value]) => {
            result[key] = mapFunction(value);
            return result;
        });
    }
    /**
     * Removes a key/value from this object and return a new object (and type)
     * To delete a (nullable) key from an object while preserving its type, use "update()" instead.
     */
    remove(keyToRemove) {
        const result = this._clone();
        delete result[keyToRemove.toString()];
        return new ObjectWrapper(result);
    }
    /**
     * Creates an Array with all these object's values.
     */
    values() {
        return this.pipe(Object.values);
    }
    /**
     * Make mutable modifications to a draft then return a new Object.
     * Example: lift({a: 1}).update(draft => {draft.a = 10})
     */
    update(updateFunction) {
        return this.pipe(o => update(o, updateFunction));
    }
    /**
     * Converts this Object to an Array of tuples.
     * Similar to Object.entries() but retains the type of keys.
     */
    toArray() {
        return this.pipe(Object.entries);
    }
    /**
     * Transforms this Object to a Map where the keys are the string typed keys of this Object.
     */
    toMap() {
        return this.pipe(o => new Map(Object.entries(o)));
    }
}
let pipe;
export function setObjectPipe(_pipe) {
    pipe = _pipe;
}
